close all
clear all
clc

% Design of Experiments (DOE)

% Lower/Upper Bounds for DOE
p1_min = 2;
p1_max = 5;
p2_min = 2;
p2_max = 5;

% Parameters for Full-Factorial
levels_p1=5;
levels_p2=10;

% Parameters for Central Composite
%type='circumscribed';
type='inscribed';
%type='faced';

% Parameters for Latin Hypercube
N_Sampl_LH=50;

% Parameters for Sobol
N_Sampl_S=50;
skip_S=3e3;
leap_S=0;

% Parameters for Halton
skip_H=3e3;
leap_H=0;
N_Sampl_HT=50;

% Parameters for Random
N_Sampl_Ran=50;


% SELECTOR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DOE_Criterion_Selector = input ('Type 1 for Full-Factorial, 2 for Central Composite, 3 for Latin Hypercube, 4 for Sobol, 5 for Halton, 6 for Random: ');

if (DOE_Criterion_Selector == 1) %Full-Factorial
    
FF = fullfact([levels_p1 levels_p2]);
vector_points_p1=rescale(FF(:,1));
vector_points_p2=rescale(FF(:,2));
matrix_points=[vector_points_p1 vector_points_p2];

end

if (DOE_Criterion_Selector == 2) %Central Composite
    
matrix_points = ccdesign(2,'type',type)*0.5+0.5;


end

if (DOE_Criterion_Selector == 3) %Latin Hypercube
    
matrix_points = lhsdesign(N_Sampl_LH,2);

end

if (DOE_Criterion_Selector == 4) %Sobol

Sobol= sobolset(2,'Skip',skip_S,'Leap',leap_S);
matrix_points=net(Sobol,N_Sampl_S);

end
    
if (DOE_Criterion_Selector == 5) %Halton   
     
Halton= haltonset(2,'Skip',skip_H,'Leap',leap_H);
matrix_points=net(Halton,N_Sampl_HT);

end

if (DOE_Criterion_Selector == 6) %Random   
     
matrix_points = rand([N_Sampl_Ran,2]);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

matrix_points_unique=unique(matrix_points,'rows');
DOE_points(:,1)=p1_min+(p1_max-p1_min).*matrix_points_unique(:,1);   
DOE_points(:,2)=p2_min+(p2_max-p2_min).*matrix_points_unique(:,2); 
[N_sampling,par]=size(DOE_points);


% Plot
figure ('Name','Tested Samplings')
plot(DOE_points(:,1),DOE_points(:,2),'o')
str1=sprintf('Number of samples = %d', N_sampling);
xlabel('p_{1}','FontSize',15,'FontWeight','bold','FontName','Calibri')
ylabel('p_{2}','FontSize',15,'FontWeight','bold','FontName','Calibri')
title(str1)

% Export to Excel
%xlswrite('DOE_2var.xls',DOE_points);


%% 
% Response Surface Modeling (RSM)

% Tested Points (DOE--> Design Exploration Domain)
x=DOE_points(:,1);
y=DOE_points(:,2);

% Responses from simulations (the following points are created via a function, though they normally come from the "experiments")
z=x.*exp(-x.^2-y.^2)+5.*sin(x+y);

% Fitting
%ft='linearinterp';
ft='cubicinterp';
%ft='poly23';

[myRSM,gof]=fit([x,y],z,ft,'Normalize', 'on');

%Plot
figure('Name','Final Response Surface');
p1=plot(myRSM,[x,y],z);
colormap(jet);
xlabel('p_{1}','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('p_{2}','FontSize',15,'FontWeight','bold','FontName','Calibri');
zlabel('Cost Function','FontSize',15,'FontWeight','bold','FontName','Calibri');
grid on

%% 
%Deterministic Alg. to find the minimum
close all
% starting point
x0 = [p1_min+0.5*(p1_max-p1_min),p2_min+0.5*(p2_max-p2_min)];

%options
options = optimset('Display','iter','PlotFcns',@optimplotfval,'MaxFunEvals',1000);

% optimization (x-->optimal point fval-->related function value)
[xyopt,fval] = fminsearch(myRSM,x0,options);

% Final Complete Plot
figure('Name','Response Surface & Optimal Point');
p1=plot(myRSM,[x,y],z);
colormap(jet);
xlabel('p_{1}','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('p_{2}','FontSize',15,'FontWeight','bold','FontName','Calibri');
zlabel('Cost Function','FontSize',15,'FontWeight','bold','FontName','Calibri');
grid on
hold on
plot3(xyopt(1),xyopt(2),fval,'o', 'MarkerFaceColor','r', 'MarkerSize',10)
