close all
clear all
clc

rng default % for reproducibility

% design variables x1=x(1), x2=x(2)
nvar=2;

% cost function (this example employs a scaled version of Rastrigins
% function with larger basins of attraction)
fun=@(x)costfun(x/10);

% starting point
x0 = [20,30]; %see the following code for implementation in ga

% lower/upper bounds for variables
lb=[];
ub=[];

%linear constraints (defined in the main script)
A = [];
b = [];
Aeq=[];
beq=[];

%nonlinear constraints (defined via an external function)
nonlcon =[]; 

%% Solution with ga
%options
MaxGen=200;  % if not specified --> 100*nvar
PoP_size=50; % if not specified --> 50 when nvar<= 5, 200 otherwise
MaxStallGen=50; % if not specified --> 50
Tol=1e-6; % if not specified --> 1e-6
CrossoverFrac=0.8; % if not specified --> 0.8
EliteFrac=0.05; % if not specified --> 0.05
initpop = 10*randn(20,nvar) + repmat(x0,20,1); %instead of setting x0, an initial pop is created. 
%If Pop_size is 50, 20 of each points are around x0 in the init generation.

options = optimoptions('ga','InitialPopulationMatrix',initpop,'Generations',MaxGen,'PopulationSize',PoP_size,'MaxStallGenerations',MaxStallGen,...
'EliteCount',ceil(EliteFrac*PoP_size),'CrossoverFraction',CrossoverFrac,'FunctionTolerance',Tol,'PlotFcn',{@gaplotbestf});

tic
% optimization (x-->optimal point fval-->related function value)
[xyopt_ga,fval_ga,flga,oga] = ga(fun,nvar,A,b,Aeq,beq,lb,ub,nonlcon,options);
toc

%% Solution with fmincon (Global Search Option)

problem = createOptimProblem('fmincon','objective',fun,'x0',x0);
gs = GlobalSearch;
[xyopt_g,fval_g,flg,og]=run(gs,problem);

%%
%Plot function
figure
[x,y]=meshgrid(-30:1:30,-30:1:30);
z=20+(x/10).^2+(y/10).^2-10*(cos(2*pi*x/10)+cos(2*pi*y/10));
surf(x,y,z);
hold on
plot3(xyopt_ga(1),xyopt_ga(2),fval_ga,'o', 'MarkerFaceColor','r', 'MarkerSize',10)
plot3(xyopt_g(1),xyopt_g(2),fval_g,'x', 'MarkerFaceColor','b', 'MarkerSize',15)

figure
scatter (og.funcCount,fval_g,'LineWidth',2)
hold on
scatter(oga.funccount,fval_ga,'LineWidth',2)
ylim([0 0.5]);
xlabel('Num Eval','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('fval','FontSize',15,'FontWeight','bold','FontName','Calibri');
