clear all
close all
clc

% Input Data
% Principal Dimensions
Rin=10;                        %[mm]
Rext=50;                       %[mm]
n_t=4;

% Calculated Spiral Start/End Angles & Pitch
alpha_Rext=n_t*2*pi;           %[rad]
p=Rext*2*pi/alpha_Rext;        %[mm]
alpha_Rin=2*pi*Rin/p;          %[rad]

% Spiral Shape Modeling
alpha=alpha_Rin:pi/100:alpha_Rext;
R=(p*alpha./(2*pi));                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
x= R.*cos(alpha);
y= R.*sin(alpha);

% Material Properties
E=1800;                         %[MPa]
nu=0.4;                         %[-]     
                
% Maximum Rotation
thetamax=40;                    %[deg]

% Target Stiffness
k_tar=20;                       %[Nmm/rad]

% Spiral Dimension - Upper/Lower Bounds                         
b_Sp_min=2;                     %[mm]    
b_Sp_max=10;                    %[mm]
t_Sp_min=1;                     %[mm]    
t_Sp_max=3.5;                   %[mm]


DOE_Criterion_Selector = input ('Type 1 for Latin Hypercube, 2 for Full-Factorial) : ');

% Total Samples for Latin Hypercube
N_Sampl_LH=15;

% Parameters for Full-Factorial

b_intervals=3;    % DOE intervals for b_Sp variable
t_intervals=5;    % DOE intervals for t_Sp variable


% Design Space Creation
if (DOE_Criterion_Selector == 1)
    
    matrix_b_t = lhsdesign(N_Sampl_LH,2);
    N_sampling=N_Sampl_LH;
end

if (DOE_Criterion_Selector == 2)
    FullFact=fullfact([b_intervals,t_intervals]);
    matrix_b_t(:,1)=(FullFact(:,1)-1)/(b_intervals-1);
    matrix_b_t(:,2)=(FullFact(:,2)-1)/(t_intervals-1);
    size_fullfac=size(FullFact);
    N_sampling=size_fullfac(1,1);
    vect_b=zeros(1,N_sampling);                   
    vect_t=zeros(1,N_sampling);
end

for i=1:1:N_sampling
vect_b(i)=b_Sp_min+(b_Sp_max-b_Sp_min)*matrix_b_t(i,1);  
vect_t(i)=t_Sp_min+(t_Sp_max-t_Sp_min)*matrix_b_t(i,2);   
end

figure ('Name','Tested Samplings')
plot(vect_b,vect_t,'MarkerSize',10,'Marker','x','LineWidth',1,'LineStyle','none','Color',[1 0 0])
str1=sprintf('Number of samples = %d', N_sampling);
xlim([b_Sp_min, b_Sp_max]);
ylim([t_Sp_min, t_Sp_max])
xlabel('b_{Sp} [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('t_{Sp} [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
title(str1)

%% DOE

for i=1:1:N_sampling
disp(i);

b_Sp=vect_b(i);
t_Sp=vect_t(i);

length=size(x);

%Input File for APDL
input = [Rin;Rext;E;nu;b_Sp;t_Sp;length(2);thetamax];
dlmwrite('FEA_Input.txt',input,'precision','%.4f','newline','pc','delimiter',' ');

SpPointsX(:,1)=x(1,:);
SpPointsY(:,1)=y(1,:);
dlmwrite('SpPointsX.txt',SpPointsX,'precision','%.3e','newline','pc','delimiter',' ');
dlmwrite('SpPointsY.txt',SpPointsY,'precision','%.3e','newline','pc','delimiter',' ');

% Batch Execution
[sta,cmd] = dos('FEA.bat','-echo');

% ANSYS Outputs Import
filename = 'FEA_Output.txt';
endRow = 1;
formatSpec = '%10f%11f%[^\n\r]';
fileID = fopen(filename,'r');
dataArray = textscan(fileID, formatSpec, endRow, 'Delimiter', '', 'WhiteSpace', '', 'TextType', 'string', 'ReturnOnError', false, 'EndOfLine', '\r\n');
fclose(fileID);
FEAOutput = [dataArray{1:end-1}];
clearvars filename endRow formatSpec fileID dataArray ans;

Stiff(i) = FEAOutput(1,1);
Stress(i) = FEAOutput(1,2);

% Delete Simulations files
delete('file.DSP');
delete('file.esav');
delete('file.mntr');
delete('file.rst');
delete('file0.err');
delete('file0.esav');
delete('file0.full');
delete('file0.log');
delete('file0.rst');
delete('file0.stat');
delete('file1.err');
delete('file1.esav');
delete('file1.full');
delete('file1.out');
delete('file1.rst');
delete('FEA_Input.txt');
delete('FEA_Output.txt');
delete('SpPointsX.txt');
delete('SpPointsY.txt');
delete('fea.out');

end

%% Fitting & Plot
%Stiff


%prova=fit( [vect_b', vect_t'], Stiff', 'cubicinterp');
%plot(prova);
%hold on
prova1=fit( [vect_b', vect_t'], Stiff', 'cubicinterp');
plot(prova1);


%%
ft = 'cubicinterp';
[xstiff, ystiff, zstiff] = prepareSurfaceData( vect_b, vect_t, Stiff );
fitfun1 = fit( [xstiff, ystiff], zstiff, ft, 'Normalize', 'on' );

figure( 'Name', 'Stiffness' );
p1=plot( fitfun1, [xstiff, ystiff], zstiff );
colormap(jet);
xlabel('b [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('t [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
zlabel('k [Nmm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
grid on

%Stress
ft = 'cubicinterp';
[xstress, ystress, zstress] = prepareSurfaceData( vect_b, vect_t, Stress );
fitfun2 = fit( [xstress, ystress], zstress, ft, 'Normalize', 'on' );

figure( 'Name', 'Stress' );
p2=plot( fitfun2, [xstress, ystress], zstress );
colormap(jet);
xlabel('b [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('t [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
zlabel('\sigma [MPa]','FontSize',15,'FontWeight','bold','FontName','Calibri');
grid on