clear all
clc

% Input Data 
% Principal Dimensions
Rin=10;                        %[mm]
Rext=50;                       %[mm]
n_t=4;

% Calculated Spiral Start/End Angles & Pitch
alpha_Rext=n_t*2*pi;           %[rad]
p=Rext*2*pi/alpha_Rext;        %[mm]
alpha_Rin=2*pi*Rin/p;          %[rad]

% Calculated Spiral Lenght
L_tot=(p/(4*pi))*((alpha_Rext*(1+alpha_Rext^2).^0.5)+(log(alpha_Rext+(1+alpha_Rext^2)^0.5)));
L_in=(p/(4*pi))*((alpha_Rin*(1+alpha_Rin^2).^0.5)+(log(alpha_Rin+(1+alpha_Rin^2)^0.5)));
L_Sp=L_tot-L_in;

% Material Properties
E=1800;                         %[MPa] 
                
% Maximum Rotation
thetamax=40;                    %[deg]
theta=(0:thetamax/10:thetamax)*pi/180;

% Spiral Dimension - Upper/Lower Bounds                      
b_Sp_min=2;                      %[mm]    
b_Sp_max=10;                     %[mm]
t_Sp_min=1;                      %[mm]    
t_Sp_max=3.5;                    %[mm]

b_Sp=b_Sp_min:0.1:b_Sp_max;
t_Sp=t_Sp_min:0.1:t_Sp_max;

[B,T] = meshgrid(b_Sp,t_Sp);

I_Sp=B.*T.^3/12;                  %[mm^4]

% Stiffness Evaluation
k_Sp=E*I_Sp./L_Sp;                %[Nmm/rad] 
M_Sp_max=k_Sp*thetamax*pi/180;    %[Nmm]     

% Stress Evaluation
Stress_Sp=6*M_Sp_max./(B.*T.^2);  %[MPa]


% Plot Analytical Results
figure( 'Name', 'Stiffness' );
surf(B,T,k_Sp)
xlabel('B [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('T [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
zlabel('K [Nmm/rad]','FontSize',15,'FontWeight','bold','FontName','Calibri');
grid on


figure( 'Name', 'Von Mises Stress' );
surf(B,T,Stress_Sp)
xlabel('B [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
ylabel('T [mm]','FontSize',15,'FontWeight','bold','FontName','Calibri');
zlabel('{\sigma}_{Max} [MPa]','FontSize',15,'FontWeight','bold','FontName','Calibri');
grid on


